==> BREWasm-1.0.0.tar.gz: BREWasm-1.0.0/README.md <==
#   B R E W a s m 
 
 
 
 B R E W a s m :   A   g e n e r a l   p u r p o s e   s t a t i c   b i n a r y   r e w r i t i n g   f r a m e w o r k   f o r   W a s m ,   w h i c h   a i m s   a t   r e d u c i n g   t h e   c o m p l e x i t y   o f   t h e   W a s m 
 
 b i n a r y   f o r m a t . 
 
 
 
 # #   F e a t u r e s 
 
 
 
 B R E W a s m   c o n s i s t s   o f   f o u r   k e y   c o m p o n e n t s :   t h e   W a s m   P a r s e r ,   s e c t i o n   r e w r i t e r ,   s e m a n t i c s   r e w r i t e r ,   a n d   W a s m   E n c o d e r .   T h e 
 
 W a s m   p a r s e r   a n d   e n c o d e r   a r e   b u i l t   u s i n g   o u r   a b s t r a c t i o n   o f   t h e   W a s m   b i n a r y ,   w h i c h   i s   r e p r e s e n t e d   a s   a   f o r m a l   f o r m a t 
 
 c o m p r i s i n g   a   l i s t   o f   o b j e c t s .   T h e   p a r s e r   a n d   e n c o d e r   e f f e c t i v e l y   c o n v e r t   t h e   W a s m   b i n a r y   a n d   a n   a r r a y   o f   o b j e c t s   i n t o 
 
 e a c h   o t h e r ,   w i t h   e a c h   o b j e c t   c o n t a i n i n g   m u l t i p l e   a t t r i b u t e s . 
 
 
 
 < d i v   a l i g n = c e n t e r > 
 
 < i m g   s r c = " d o c / D e f i n i t i o n . p n g "   w i d t h = " 6 0 0 " > 
 
     < d i v   s t y l e = " m a r g i n - t o p :   1 0 p x ;   m a r g i n - b o t t o m :   1 0 p x " > 
 
         < b > F o r m a l   d e f i n i t i o n   o f   s e c t i o n s ,   e l e m e n t s   a n d   f i e l d s   i n   W a s m . < / b > 
 
     < / d i v > 
 
 < / d i v >     
 
 
 
 -   T h e   s e c t i o n   r e w r i t e r   d i r e c t l y   i n t e r a c t s   w i t h   t h e   f o r m a l   d e f i n i t i o n ,   e . g . ,   i n s e r t i n g / d e l e t i n g   a   n e w   o b j e c t   o r   m o d i f y i n g 
 
     a t t r i b u t e s   o f   e x i s t i n g   o b j e c t s .   I t   p a c k s   t h e s e   f i n e - g r a i n e d   r e w r i t i n g   f u n c t i o n s   i n t o   A P I s . 
 
 -   T h e   s e m a n t i c s   r e w r i t e r   f u r t h e r   c o m b i n e s   t h e   f i n e - g r a i n e d   A P I s   o f   s e c t i o n   r e w r i t e r   a n d   o f f e r s   a n o t h e r   s e t   o f   h i g h - l e v e l 
 
     A P I s ,   w h e r e   e a c h   o f   t h e m   p o s s e s s e s   r i c h   s e m a n t i c s   a s   f o l l o w i n g ,   l i k e   i n s e r t i n g   a   f u n c t i o n ,   a n d   a p p e n d   a   p i e c e   o f 
 
     l i n e a r   m e m o r y . 
 
         -   G l o b a l   V a r i a b l e s 
 
         -   I m p o r t   &   E x p o r t 
 
         -   L i n e a r   M e m o r y 
 
         -   F u n c t i o n 
 
         -   C u s t o m   C o n t e n t 
 
 
 
 # #   I n s t a l l a t i o n 
 
 
 
 # # #   P y t h o n   p a c k a g e 
 
 
 
 B R E W a s m   i s   c u r r e n t l y   a v a i l a b l e   o n   P I P   r e p o s i t o r i e s . 
 
 
 
 I n s t a l l   B R E W a s m : : 
 
 
 
 ` ` ` 
 
 p i p   i n s t a l l   B R E W a s m 
 
 ` ` ` 
 
 
 
 
 
 # #   E x a m p l e s 
 
 
 
 # # #   S e c t i o n   R e w r i t e r 
 
 
 
 T h e   b a s i c   o p e r a t i o n   o f   t h e   s e c t i o n   r e w r i t e r   i s   ` s e l e c t ` ,   ` i n s e r t ` ,   ` u p d a t e `   a n d   ` d e l e t e ` . 
 
 
 
 ` ` ` p y t h o n 
 
 #   - - - - - - - - - -   s o m e _ t e s t s . p y   - - - - - - - - - - 
 
 f r o m   B R E W a s m   i m p o r t   * 
 
 
 
 b i n a r y   =   B R E W a s m ( ' a . w a s m ' )     #   O p e n   a   W a s m   b i n a r y   f i l e 
 
 
 
 #   I n i t i a l i z e   a   s e c t i o n   r e w r i t e r   o f   t h e   g l o b a l   s e c t i o n .   
 
 g l o b a l _ r e w r i t e r   =   S e c t i o n R e w r i t e r ( s e l f . b i n a r y . m o d u l e ,   s e l f . b i n a r y . m o d u l e . g l o b a l _ s e c ) 
 
 
 
 #   S e l e c t   a l l   t h e   i t e m s   i n   g l o b a l   s e c t i o n 
 
 g l o b a l _ l i s t   =   g l o b a l _ r e w r i t e r . s e l e c t ( ) 
 
 #   G e t   t h e   a t t r i b u t e   g l o b a l i d x   o f   a   g l o b a l   i t e m ,   w h o s e   i n d e x   i s   o n e . 
 
 i d x   =   g l o b a l _ l i s t [ 1 ] . g l o b a l i d x 
 
 #   I n s e r t   a   n e w   g l o b a l   i t e m   a t   t h e   i n d e x   i d x   o f   t h e   g l o b a l   s e c t i o n 
 
 g l o b a l _ r e w r i t e r . i n s e r t ( G l o b a l ( i d x ) ,   G l o b a l ( v a l t y p e = I 3 2 ,   v a l = 1 0 0 ) ) 
 
 #   D e l e t e   t h e   g l o b a l   i t e m   w h o s e   i n d e x   i s   i d x . 
 
 g l o b a l _ r e w r i t e r . d e l e t e ( G l o b a l ( i d x ) ) 
 
 #   E m i t   a   n e w   b i n a r y   f i l e 
 
 b i n a r y . e m i t _ b i n a r y ( ' b . w a s m ' ) 
 
 ` ` ` 
 
 
 
 # # #   S e m a n t i c s   R e w r i t e r 
 
 
 
 ` ` ` p y t h o n 
 
 f r o m   B R E W a s m   i m p o r t   * 
 
 
 
 b i n a r y   =   B R E W a s m ( ' a . w a s m ' )     #   O p e n   a   W a s m   b i n a r y   f i l e 
 
 
 
 #   I n i t i a l i z e   a   s e m a n t i c s   r e w r i t e r   o f   t h e   f u n c t i o n   s e m a n t i c s 
 
 f u n c t i o n _ r e w r i t e r   =   S e m a n t i c R e w r i t e r . F u n c t i o n ( b i n a r y ) 
 
 #   D e f i n e   t h e   i n s t r u c t i o n s   o f   f u n c t i o n 
 
 f u n c b o d y   =   [ I n s t r u c t i o n ( L o c a l G e t ,   0 ) ,   I n s t r u c t i o n ( L o c a l G e t ,   1 ) ,   I n s t r u c t i o n ( I 3 2 A d d ,   0 ) ,   I n s t r u c t i o n ( N o p ) ] 
 
 #   I n s e r t   a   i n t e r n a l   f u n c t i o n   i n   t h e   b i n a r y 
 
 f u n c t i o n _ r e w r i t e r . i n s e r t _ i n t e r n a l _ f u n c t i o n ( i d x = 1 ,   p a r a m s _ t y p e = [ I 3 2 ,   I 3 2 ] ,   r e s u l t s _ t y p e = [ I 3 2 ] , 
 
                                                                                       l o c a l _ v e c = [ L o c a l ( 0 ,   I 3 2 ) ,   L o c a l ( 1 ,   I 6 4 ) ] ,   f u n c b o d y ) 
 
 #   E m i t   a   n e w   b i n a r y   f i l e 
 
 b i n a r y . e m i t _ b i n a r y ( ' b . w a s m ' ) 
 
 ` ` ` 
 
 
 
 # #   D o c u m e n t a t i o n 
 
 
 
 T h e   c o m p l e t e   d o c u m e n t a t i o n   c a n   b e   f o u n d   [ h e r e ] ( h t t p s : / / b r e w a s m - p r o j e c t . r e a d t h e d o c s . i o / e n / l a t e s t / ) . 

==> BREWasm-1.0.1.tar.gz: BREWasm-1.0.1/README.md <==
#   B R E W a s m 
 
 
 
 B R E W a s m :   A   g e n e r a l   p u r p o s e   s t a t i c   b i n a r y   r e w r i t i n g   f r a m e w o r k   f o r   W a s m ,   w h i c h   a i m s   a t   r e d u c i n g   t h e   c o m p l e x i t y   o f   t h e   W a s m 
 
 b i n a r y   f o r m a t . 
 
 
 
 # #   F e a t u r e s 
 
 
 
 B R E W a s m   c o n s i s t s   o f   f o u r   k e y   c o m p o n e n t s :   t h e   W a s m   P a r s e r ,   s e c t i o n   r e w r i t e r ,   s e m a n t i c s   r e w r i t e r ,   a n d   W a s m   E n c o d e r .   T h e 
 
 W a s m   p a r s e r   a n d   e n c o d e r   a r e   b u i l t   u s i n g   o u r   a b s t r a c t i o n   o f   t h e   W a s m   b i n a r y ,   w h i c h   i s   r e p r e s e n t e d   a s   a   f o r m a l   f o r m a t 
 
 c o m p r i s i n g   a   l i s t   o f   o b j e c t s .   T h e   p a r s e r   a n d   e n c o d e r   e f f e c t i v e l y   c o n v e r t   t h e   W a s m   b i n a r y   a n d   a n   a r r a y   o f   o b j e c t s   i n t o 
 
 e a c h   o t h e r ,   w i t h   e a c h   o b j e c t   c o n t a i n i n g   m u l t i p l e   a t t r i b u t e s . 
 
 
 
 < d i v   a l i g n = c e n t e r > 
 
 < i m g   s r c = " d o c / D e f i n i t i o n . p n g "   w i d t h = " 6 0 0 " > 
 
     < d i v   s t y l e = " m a r g i n - t o p :   1 0 p x ;   m a r g i n - b o t t o m :   1 0 p x " > 
 
         < b > F o r m a l   d e f i n i t i o n   o f   s e c t i o n s ,   e l e m e n t s   a n d   f i e l d s   i n   W a s m . < / b > 
 
     < / d i v > 
 
 < / d i v >     
 
 
 
 -   T h e   s e c t i o n   r e w r i t e r   d i r e c t l y   i n t e r a c t s   w i t h   t h e   f o r m a l   d e f i n i t i o n ,   e . g . ,   i n s e r t i n g / d e l e t i n g   a   n e w   o b j e c t   o r   m o d i f y i n g 
 
     a t t r i b u t e s   o f   e x i s t i n g   o b j e c t s .   I t   p a c k s   t h e s e   f i n e - g r a i n e d   r e w r i t i n g   f u n c t i o n s   i n t o   A P I s . 
 
 -   T h e   s e m a n t i c s   r e w r i t e r   f u r t h e r   c o m b i n e s   t h e   f i n e - g r a i n e d   A P I s   o f   s e c t i o n   r e w r i t e r   a n d   o f f e r s   a n o t h e r   s e t   o f   h i g h - l e v e l 
 
     A P I s ,   w h e r e   e a c h   o f   t h e m   p o s s e s s e s   r i c h   s e m a n t i c s   a s   f o l l o w i n g ,   l i k e   i n s e r t i n g   a   f u n c t i o n ,   a n d   a p p e n d   a   p i e c e   o f 
 
     l i n e a r   m e m o r y . 
 
         -   G l o b a l   V a r i a b l e s 
 
         -   I m p o r t   &   E x p o r t 
 
         -   L i n e a r   M e m o r y 
 
         -   F u n c t i o n 
 
         -   C u s t o m   C o n t e n t 
 
 
 
 # #   I n s t a l l a t i o n 
 
 
 
 # # #   P y t h o n   p a c k a g e 
 
 
 
 B R E W a s m   i s   c u r r e n t l y   a v a i l a b l e   o n   P I P   r e p o s i t o r i e s . 
 
 
 
 I n s t a l l   B R E W a s m : : 
 
 
 
 ` ` ` 
 
 p i p   i n s t a l l   B R E W a s m 
 
 ` ` ` 
 
 
 
 
 
 # #   E x a m p l e s 
 
 
 
 # # #   S e c t i o n   R e w r i t e r 
 
 
 
 T h e   b a s i c   o p e r a t i o n   o f   t h e   s e c t i o n   r e w r i t e r   i s   ` s e l e c t ` ,   ` i n s e r t ` ,   ` u p d a t e `   a n d   ` d e l e t e ` . 
 
 
 
 ` ` ` p y t h o n 
 
 #   - - - - - - - - - -   s o m e _ t e s t s . p y   - - - - - - - - - - 
 
 f r o m   B R E W a s m   i m p o r t   * 
 
 
 
 b i n a r y   =   B R E W a s m ( ' a . w a s m ' )     #   O p e n   a   W a s m   b i n a r y   f i l e 
 
 
 
 #   I n i t i a l i z e   a   s e c t i o n   r e w r i t e r   o f   t h e   g l o b a l   s e c t i o n .   
 
 g l o b a l _ r e w r i t e r   =   S e c t i o n R e w r i t e r ( s e l f . b i n a r y . m o d u l e ,   g l o b a l s e c = s e l f . b i n a r y . m o d u l e . g l o b a l _ s e c ) 
 
 
 
 #   S e l e c t   a l l   t h e   i t e m s   i n   g l o b a l   s e c t i o n 
 
 g l o b a l _ l i s t   =   g l o b a l _ r e w r i t e r . s e l e c t ( G l o b a l ( ) ) 
 
 #   G e t   t h e   a t t r i b u t e   g l o b a l i d x   o f   a   g l o b a l   i t e m ,   w h o s e   i n d e x   i s   o n e . 
 
 i d x   =   g l o b a l _ l i s t [ 1 ] . g l o b a l i d x 
 
 #   I n s e r t   a   n e w   g l o b a l   i t e m   a t   t h e   i n d e x   i d x   o f   t h e   g l o b a l   s e c t i o n 
 
 g l o b a l _ r e w r i t e r . i n s e r t ( G l o b a l ( i d x ) ,   G l o b a l ( v a l t y p e = I 3 2 ,   v a l = 1 0 0 ) ) 
 
 #   D e l e t e   t h e   g l o b a l   i t e m   w h o s e   i n d e x   i s   i d x . 
 
 g l o b a l _ r e w r i t e r . d e l e t e ( G l o b a l ( i d x ) ) 
 
 #   E m i t   a   n e w   b i n a r y   f i l e 
 
 b i n a r y . e m i t _ b i n a r y ( ' b . w a s m ' ) 
 
 ` ` ` 
 
 
 
 # # #   S e m a n t i c s   R e w r i t e r 
 
 
 
 ` ` ` p y t h o n 
 
 f r o m   B R E W a s m   i m p o r t   * 
 
 
 
 b i n a r y   =   B R E W a s m ( ' a . w a s m ' )     #   O p e n   a   W a s m   b i n a r y   f i l e 
 
 
 
 #   I n i t i a l i z e   a   s e m a n t i c s   r e w r i t e r   o f   t h e   f u n c t i o n   s e m a n t i c s 
 
 f u n c t i o n _ r e w r i t e r   =   S e m a n t i c R e w r i t e r . F u n c t i o n ( b i n a r y . m o d u l e ) 
 
 #   D e f i n e   t h e   i n s t r u c t i o n s   o f   f u n c t i o n 
 
 f u n c b o d y   =   [ I n s t r u c t i o n ( L o c a l G e t ,   0 ) ,   I n s t r u c t i o n ( L o c a l G e t ,   1 ) ,   I n s t r u c t i o n ( I 3 2 A d d ,   0 ) ,   I n s t r u c t i o n ( N o p ) ] 
 
 #   I n s e r t   a   i n t e r n a l   f u n c t i o n   i n   t h e   b i n a r y 
 
 f u n c t i o n _ r e w r i t e r . i n s e r t _ i n t e r n a l _ f u n c t i o n ( i d x = 1 ,   p a r a m s _ t y p e = [ I 3 2 ,   I 3 2 ] ,   r e s u l t s _ t y p e = [ I 3 2 ] , 
 
                                                                                       l o c a l _ v e c = [ L o c a l ( 0 ,   I 3 2 ) ,   L o c a l ( 1 ,   I 6 4 ) ] ,   f u n c b o d y ) 
 
 #   E m i t   a   n e w   b i n a r y   f i l e 
 
 b i n a r y . e m i t _ b i n a r y ( ' b . w a s m ' ) 
 
 ` ` ` 
 
 
 
 # #   D o c u m e n t a t i o n 
 
 
 
 T h e   c o m p l e t e   d o c u m e n t a t i o n   c a n   b e   f o u n d   [ h e r e ] ( h t t p s : / / b r e w a s m - p r o j e c t . r e a d t h e d o c s . i o / e n / l a t e s t / ) . 
